@@ -59,12 +59,12 @@ void cache_init(int size, int logq)
     hash_table[i] = NULL;
 }
 
-static struct crec **hash_bucket(unsigned char *name)
+static struct crec **hash_bucket(char *name)
 {
   unsigned int c, val = 0;
   
   /* don't use tolower and friends here - they may be messed up by LOCALE */
-  while((c = *name++))
+  while((c = (unsigned char) *name++))
     if (c >= 'A' && c <= 'Z')
       val += c + 'a' - 'A';
     else
@@ -562,7 +562,7 @@ static void read_hostsfile(char *filename, int opts, char *buff, char *domain_su
   
   if (!f)
     {
-      syslog(LOG_ERR, "failed to load names from %s: %m", filename);
+      syslog(LOG_ERR, _("failed to load names from %s: %m"), filename);
       return;
     }
     
@@ -579,12 +579,12 @@ static void read_hostsfile(char *filename, int opts, char *buff, char *domain_su
 	continue;
 
 #ifdef HAVE_IPV6      
-      if (inet_pton(AF_INET, token, &addr) == 1)
+      if (inet_pton(AF_INET, token, &addr) > 0)
 	{
 	  flags = F_HOSTS | F_IMMORTAL | F_FORWARD | F_REVERSE | F_IPV4;
 	  addrlen = INADDRSZ;
 	}
-      else if (inet_pton(AF_INET6, token, &addr) == 1)
+      else if (inet_pton(AF_INET6, token, &addr) > 0)
 	{
 	  flags = F_HOSTS | F_IMMORTAL | F_FORWARD | F_REVERSE | F_IPV6;
 	  addrlen = IN6ADDRSZ;
@@ -597,7 +597,10 @@ static void read_hostsfile(char *filename, int opts, char *buff, char *domain_su
 	}
 #endif
       else
-	continue;
+	{
+	  syslog(LOG_ERR, _("bad address at %s line %d"), filename, lineno); 
+	  continue;
+	}
 
      while ((token = strtok(NULL, " \t\n\r")) && (*token != '#'))
        {
@@ -622,13 +625,13 @@ static void read_hostsfile(char *filename, int opts, char *buff, char *domain_su
 	       }
 	   }
 	 else
-	   syslog(LOG_ERR, "bad name at %s line %d", filename, lineno); 
+	   syslog(LOG_ERR, _("bad name at %s line %d"), filename, lineno); 
        }
     }
   
   fclose(f);
 
-  syslog(LOG_INFO, "read %s - %d addresses", filename, count);
+  syslog(LOG_INFO, _("read %s - %d addresses"), filename, count);
 }
 	    
 void cache_reload(int opts, char *buff, char *domain_suffix, struct hostsfile *addn_hosts)
@@ -664,7 +667,7 @@ void cache_reload(int opts, char *buff, char *domain_suffix, struct hostsfile *a
   if ((opts & OPT_NO_HOSTS) && !addn_hosts)
     {
       if (cache_size > 0)
-	syslog(LOG_INFO, "cleared cache");
+	syslog(LOG_INFO, _("cleared cache"));
       return;
     }
 
@@ -717,8 +720,8 @@ void cache_add_dhcp_entry(struct daemon *daemon, char *host_name,
 	    {
 	      strcpy(daemon->namebuff, inet_ntoa(crec->addr.addr.addr.addr4));
 	      syslog(LOG_WARNING, 
-		     "not giving name %s to the DHCP lease of %s because "
-		     "the name exists in %s with address %s", 
+		     _("not giving name %s to the DHCP lease of %s because "
+		       "the name exists in %s with address %s"), 
 		     host_name, inet_ntoa(*host_address),
 		     record_source(daemon->addn_hosts, crec->uid), daemon->namebuff);
 	    }
@@ -761,7 +764,7 @@ void cache_add_dhcp_entry(struct daemon *daemon, char *host_name,
 
 void dump_cache(struct daemon *daemon)
 {
-  syslog(LOG_INFO, "cache size %d, %d/%d cache insertions re-used unexpired cache entries.", 
+  syslog(LOG_INFO, _("cache size %d, %d/%d cache insertions re-used unexpired cache entries."), 
 	 daemon->cachesize, cache_live_freed, cache_inserted); 
   
   if (daemon->options & (OPT_DEBUG | OPT_LOG))
@@ -902,9 +905,9 @@ void log_query(unsigned short flags, char *name, struct all_addr *addr,
   else if (flags & F_QUERY)
     {
       unsigned int i;
-      static struct {
+      static const struct {
 	unsigned int type;
-	char *name;
+	const char * const name;
       } typestr[] = {
 	{ 1,   "A" },
 	{ 2,   "NS" },
@@ -948,6 +951,9 @@ void log_query(unsigned short flags, char *name, struct all_addr *addr,
   else
     source = "cached";
   
+  if (strlen(name) == 0)
+    name = ".";
+
   if ((flags & F_FORWARD) | (flags & F_NEG))
     syslog(LOG_DEBUG, "%s %s %s %s", source, name, verb, addrbuff);
   else if (flags & F_REVERSE)
